package com.thomaskuenneth.sages;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Locale;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.speech.tts.TextToSpeech.OnUtteranceCompletedListener;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;

/**
 * Diese Activity liest einen eingegebenen Text in einer der zur Verfgung
 * stehenden Sprachen vor.
 * 
 * @author Thomas Knneth
 * 
 */
public class SagEs extends Activity implements OnInitListener,
		OnUtteranceCompletedListener {

	private static final int RQ_CHECK_TTS_DATA = 0x4242;

	private TextToSpeech tts;
	private Hashtable<String, Locale> htCountryLocale;
	private String last_utterance_id;

	private EditText inputView;
	private Spinner localeView;
	private Button stopButton;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// die Sprachsynthesekomponente wurde noch nicht initialisiert
		tts = null;
		// prfen, ob Sprachpakete vorhanden sind
		Intent checkIntent = new Intent();
		checkIntent.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
		startActivityForResult(checkIntent, RQ_CHECK_TTS_DATA);
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		if (tts != null) {
			// Ressourcen freigeben
			tts.shutdown();
		}
	}

	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		// Sind Sprachpakete vorhanden?
		if (requestCode == RQ_CHECK_TTS_DATA) {
			if (resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS) {
				// ja: Activity initialisieren
				setContentView(R.layout.main);
				inputView = (EditText) findViewById(R.id.input);
				localeView = (Spinner) findViewById(R.id.locale);
				Button sayitButton = (Button) findViewById(R.id.say_it);
				sayitButton.setOnClickListener(new OnClickListener() {

					@Override
					public void onClick(View v) {
						String text = inputView.getText().toString();
						String key = (String) localeView.getSelectedItem();
						Locale loc = htCountryLocale.get(key);
						if (loc != null) {
							tts.setLanguage(loc);
//							tts.setLanguage(Locale.US);
//							tts.setLanguage(Locale.UK);
							stopButton.setEnabled(true);
							HashMap<String, String> hashmap = new HashMap<String, String>();
							last_utterance_id = Long.toString(System
									.currentTimeMillis());
							hashmap.put(
									TextToSpeech.Engine.KEY_PARAM_UTTERANCE_ID,
									last_utterance_id);
							tts.speak(text, TextToSpeech.QUEUE_FLUSH, hashmap);
						}
					}
				});
				stopButton = (Button) findViewById(R.id.stop);
				stopButton.setOnClickListener(new OnClickListener() {

					@Override
					public void onClick(View v) {
						tts.stop();
					}
				});

				// Initialisierung der Sprachkomponente starten
				tts = new TextToSpeech(this, this);
			} else {
				// nein, daher Installation der Sprachpakete vorbereiten
				Intent installIntent = new Intent();
				installIntent
						.setAction(TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
				startActivity(installIntent);
				// Activity beenden
				finish();
			}
		}
	}

	// OnInitListener

	@Override
	public void onInit(int status) {
		if (status != TextToSpeech.SUCCESS) {
			// die Initialisierung war nicht erfolgreich
			finish();
		}
		tts.setOnUtteranceCompletedListener(this);
		htCountryLocale = new Hashtable<String, Locale>();
		String[] languages = Locale.getISOLanguages();
		for (String lang : languages) {
			Locale loc = new Locale(lang);
			switch (tts.isLanguageAvailable(loc)) {
			case TextToSpeech.LANG_MISSING_DATA:
			case TextToSpeech.LANG_NOT_SUPPORTED:
				break;
			default:
				String key = loc.getDisplayLanguage();
				if (!htCountryLocale.containsKey(key)) {
					htCountryLocale.put(key, loc);
				}
				break;
			}
		}
		ArrayAdapter<Object> adapter = new ArrayAdapter<Object>(this,
				android.R.layout.simple_spinner_item, htCountryLocale.keySet()
						.toArray());
		adapter
				.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		localeView.setAdapter(adapter);
	}

	@Override
	public void onUtteranceCompleted(final String utteranceId) {
		final Handler h = new Handler(Looper.getMainLooper());
		h.post(new Runnable() {

			@Override
			public void run() {
				if (utteranceId.equals(last_utterance_id)) {
					stopButton.setEnabled(false);
				}
			}

		});
	}
}